/*********************************************************************************
 * Copyright (c) 2007 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/

package de.fzj.unicore.cisprovider.impl;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.cisprovider.CISInfoProvider;
import de.fzj.unicore.cisprovider.Config;
import de.fzj.unicore.cisprovider.Monitor;
import de.fzj.unicore.cisprovider.sensors.ApplicationsInfo;
import de.fzj.unicore.cisprovider.sensors.ExecutionEnvironmentInfo;
import de.fzj.unicore.cisprovider.sensors.JobInfo;
import de.fzj.unicore.cisprovider.sensors.SiteInfo;
import de.fzj.unicore.cisprovider.sensors.TextInfo;
import de.fzj.unicore.uas.impl.registry.RegistryHandler;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xfire.XFireKernel;

public class InitOnStartup implements Runnable{

	private final Kernel kernel;
	
	public InitOnStartup(Kernel kernel){
		this.kernel=kernel;
	}
	
	/**
	 * property for disabling publishing CIP to registry
	 */
	public static final String PUBLISH="de.fzj.unicore.cisprovider.publish";

	public void run(){
		if(!cipAvailable()){
			CISInfoProviderImpl.logger.warn("CIS Infoprovider is not deployed, " +
			"but startup code was configured. Please check the configuration.");
			return;
		}
		Monitor m=Config.get(kernel).getMonitor();
		m.addSensor(new SiteInfo(kernel));
		m.addSensor(new JobInfo(kernel));
		m.addSensor(new ApplicationsInfo(kernel));
		m.addSensor(new ExecutionEnvironmentInfo(kernel));
		m.addSensor(new TextInfo(kernel));
		publish();
	}

	protected boolean cipAvailable(){
		return XFireKernel.get(kernel).getService(CISInfoProvider.SERVICENAME)!=null;
	}
	
	/**
	 * publishes CIP web service in the registry <br/>
	 */
	protected void publish() {
		try {
			//check if publishing is disabled
			if(!kernel.getBooleanProperty(PUBLISH, true)){
				CISInfoProviderImpl.logger.info("NOT publishing CIP to registry.");
				return;
			}
			EndpointReferenceType epr = WSServerUtilities.newEPR(kernel);
			epr.addNewAddress().setStringValue(WSServerUtilities.makeAddress(CISInfoProvider.SERVICENAME, kernel));
			WSServerUtilities.addPortType(epr, CISInfoProvider.PORTTYPE);
			RegistryHandler.get(kernel).getRegistryClient().addRegistryEntry(epr);
			CISInfoProviderImpl.logger.info("Added registry entry for " + CISInfoProvider.SERVICENAME);

		} catch (Exception e) {
			LogUtil.logException("Could not publish to registry", e, CISInfoProviderImpl.logger);
		}
	}


}
