package de.fzj.unicore.bes.util;

import java.io.File;

import javax.xml.namespace.QName;

import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlObject;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ogf.schemas.hpcp.x2007.x01.bp.BasicFilterDocument;
import org.ogf.schemas.hpcp.x2007.x01.bp.BasicFilterType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.ReferenceParametersType;
import org.xmlsoap.schemas.soap.envelope.Fault;

public class ActivityUtils {
	
	public static Fault buildSOAP11Fault(QName faultCode, String faultString) {
		Fault soapFault = Fault.Factory.newInstance();
		// when policies will be applied then appropriate error message will be
		// generated
		soapFault.setFaultcode(faultCode);
		soapFault.setFaultstring(faultString);
		return soapFault;
	}

	public static XmlObject filterActivityAndResources() {
		BasicFilterDocument bfDoc = BasicFilterDocument.Factory.newInstance();
		BasicFilterType bfType = bfDoc.addNewBasicFilter();
		bfType.setActivityReferences(true);
		bfType.setContainedResources(true);
		return bfDoc;
	}

	public static XmlObject filterOnlyActivityBeans() {
		BasicFilterDocument bfDoc = BasicFilterDocument.Factory.newInstance();
		BasicFilterType bfType = bfDoc.addNewBasicFilter();
		bfType.setActivityReferences(true);
		bfType.setContainedResources(false);
		return bfDoc;
	}

	public static XmlObject filterOnlyResourceBeans() {
		BasicFilterDocument bfDoc = BasicFilterDocument.Factory.newInstance();
		BasicFilterType bfType = bfDoc.addNewBasicFilter();
		bfType.setActivityReferences(false);
		bfType.setContainedResources(true);
		return bfDoc;
	}

	public static XmlObject filterNoneBeans() {
		BasicFilterDocument bfDoc = BasicFilterDocument.Factory.newInstance();
		BasicFilterType bfType = bfDoc.addNewBasicFilter();
		bfType.setActivityReferences(false);
		bfType.setContainedResources(false);
		return bfDoc;
	}

	/**
	 * This method extracts the incoming epr's resourceid within the reference parameters. 
	 *  
	 * @return referenceParam 
	 * */
	public static String extractRefParamResourceId(EndpointReferenceType eprt) {
		ReferenceParametersType refParam = eprt.getReferenceParameters();
		if(refParam == null) return null;
		XmlCursor cursor = refParam.newCursor();
		cursor.toFirstChild();
		String rid =  cursor.getTextValue();
		cursor.dispose();
		return rid;
	}
	
	public static JobDefinitionDocument getJSDLFromFile(String filePath) throws Exception{
			JobDefinitionDocument jobDoc = null;
			jobDoc = JobDefinitionDocument.Factory.parse(new File(filePath));
			return jobDoc;
			
	}

	public static void deleteFile(File f){
		if (!f.isDirectory()) {
			f.delete();
		} else {
			for (File child : f.listFiles()) {
				deleteFile(child);
			}
		}
	}
	
}
