/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licensing information.
 *
 * Author: T. Rękawek <newton@mat.uni.torun.pl>
 */

package de.fzj.unicore.uas.impl.dsms;

import java.util.Calendar;

import org.apache.log4j.Logger;

import pl.edu.icm.dsms.catalogue.CatalogueClient;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Kernel;
import eu.unicore.security.util.Log;

/**
 * Singleton that gets free space info from dCatalogue.
 * @author newton
 *
 */
public final class FreeSpaceChecker
{
	private static final Logger logger = Log.getLogger(LogUtil.SERVICES, FreeSpaceChecker.class);
	
	private static int checkFreq = 5;
	
	private Long lastCheckTime = null;
	private Long lastValue = null;
	private CatalogueClient cat = null;
	private boolean checkFreeSpace;
	private final Kernel kernel;
	
    public static synchronized FreeSpaceChecker get(Kernel kernel)
    {
    	FreeSpaceChecker instance=kernel.getAttribute(FreeSpaceChecker.class);
        if (instance == null){
                  instance = new FreeSpaceChecker(kernel);
                  kernel.setAttribute(FreeSpaceChecker.class, instance);
            }
        return instance;
    }
 
    private FreeSpaceChecker(Kernel kernel)
    {
    	this.kernel=kernel;
    	checkFreeSpace=kernel.getBooleanProperty(DSMSImpl.GET_FREE_SPACE, true);
    }
 
    synchronized public Long getFreeSpace()
    {
    	if(!checkFreeSpace)
    		return null;
    	
    	long now = Calendar.getInstance().getTimeInMillis();
    	if(lastCheckTime == null || (now - lastCheckTime) > checkFreq * 1000)
    	{
    		lastCheckTime = now;
    		try
    		{
    			lastValue = getCatalogue().getFreeSpace();
    		}
    		catch(Exception e)
    		{
    			logger.error("Can't get free space");
    			lastValue = null;
    		}
    	}
    	return lastValue;
    }

	private CatalogueClient getCatalogue() throws Exception
	{
		try
		{
			if(cat != null && cat.getCurrentTime() != null)
				return cat;
		}
		catch(Exception e)
		{
			logger.debug("Catalogue client is no longer valid.", e);
		}
		try
		{
			return cat = Utils.getCatalogueClientFromRegistry(null,kernel);
		}
		catch(Exception e)
		{
			logger.debug("Can't get dCatalogue");
			throw e;
		}
	}
}