/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/
 

package de.fzj.unicore.uas.impl.registry;

import de.fzj.unicore.persist.PersistenceException;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.xmlbeans.impl.WSResourceHomeImpl;
import de.fzj.unicore.wsrflite.xmlbeans.impl.WSResourceImpl;

public class LocalRegistryEntryHomeImpl extends WSResourceHomeImpl {

	/**
	 * setup the expiry check
	 * Local registry entries do not expire but become refreshed when they reach 
	 * their termination time.
	 */
	protected void initExpiryCheck()throws Exception{
		super.initExpiryCheck();
		instanceChecking.removeChecker(expiryChecker);
		expiryChecker = new RegistryEntryUpdater();
		instanceChecking.addChecker(expiryChecker);
	}
		
	@Override
	protected WSResourceImpl doCreateInstance() {
		return new RegistryEntryImpl();
	}

	@Override
	public void runExpiryCheckNow(){
		super.runExpiryCheckNow();
		try{
			for(String id: serviceInstances.getUniqueIDs()){
				try{

					//forces refresh of registry entry in global registry
					expiryChecker.process(this, id);
				}catch(Exception ex){
					LogUtil.logException("Error refreshing registry entry.", ex, logger);
				}
			}
		}catch(PersistenceException pe){
			throw new RuntimeException(pe);
		}
	}

}
