/*********************************************************************************
 * Copyright (c) 2006 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/

package de.fzj.unicore.uas.impl.registry;

import org.oasisOpen.docs.wsrf.rl2.DestroyDocument;
import org.oasisOpen.docs.wsrf.rl2.DestroyResponseDocument;
import org.oasisOpen.docs.wsrf.sg2.ServiceGroupEPRDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.messaging.Message;
import de.fzj.unicore.wsrflite.security.SecurityManager;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.exceptions.ResourceNotDestroyedFault;
import de.fzj.unicore.wsrflite.xmlbeans.exceptions.ResourceUnavailableFault;
import de.fzj.unicore.wsrflite.xmlbeans.exceptions.ResourceUnknownFault;
import de.fzj.unicore.wsrflite.xmlbeans.sg.impl.ServiceGroupEntryImpl;
import eu.unicore.security.Client;

/**
 * The RegistryEntryImpl extends the basic {@link ServiceGroupEntryImpl} 
 * by security and access control, to allow control over who may destroy 
 * a registry entry
 *
 * @author schuller
 */
public class RegistryEntryImpl extends ServiceGroupEntryImpl{

	@Override
	public DestroyResponseDocument Destroy(DestroyDocument dd)
	throws ResourceNotDestroyedFault, ResourceUnknownFault,
	ResourceUnavailableFault {
		//allow to destroy if client has admin permission
		Client c=getClient();
		if(c!=null){
			if(SecurityManager.isAdmin(c)|| kernel.getSecurityManager().isServer(c)){
				return super.Destroy(dd);	
			}
		}
		
		throw ResourceNotDestroyedFault.createFault("Can only be destroyed by admins");
	}

	@Override
	public void destroy() {
		try{
			logger.debug("Destroying RegistryEntry with id "+this.getUniqueID());
			EndpointReferenceType sg=((ServiceGroupEPRDocument)getResourcePropertyXML(RPServiceGroupEPR)[0]).getServiceGroupEPR();
			String id=WSServerUtilities.extractResourceID(sg);
			Message m=new Message(getUniqueID());
			kernel.getMessaging().getChannel(id).publish(m);
		}
		catch(Exception e){
			LogUtil.logException("Could not send internal message.",e,logger);
		}
	}

}
