/*********************************************************************************
 * Copyright (c) 2010 Forschungszentrum Juelich GmbH 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * (1) Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer at the end. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.
 * 
 * (2) Neither the name of Forschungszentrum Juelich GmbH nor the names of its 
 * contributors may be used to endorse or promote products derived from this 
 * software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ********************************************************************************/


package de.fzj.unicore.uas.impl.sms;

import org.apache.log4j.Logger;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.FileSystemDocument;
import org.unigrids.x2006.x04.services.smf.StorageDescriptionDocument;
import org.unigrids.x2006.x04.services.smf.StorageDescriptionType;

import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.Resource;
import de.fzj.unicore.wsrflite.security.util.AuthZAttributeStore;
import de.fzj.unicore.wsrflite.xmlbeans.ResourceProperty;
import eu.unicore.security.Client;

/**
 * publishes descriptions of the available storage types to clients
 *
 * @author schuller
 */
public class StorageDescriptionRP extends  ResourceProperty<Object>{

	private static final long serialVersionUID=1L;

	private static final Logger logger=LogUtil.getLogger(LogUtil.SERVICES, StorageDescriptionRP.class);

	StorageDescriptionDocument[] xml;

	private long lastPropertyUpdate=-1;

	private final String xnjsReference;
	
	/**
	 * property defining the available storage types
	 */
	public static final String DEFINED_TYPES=StorageFactoryImpl.PREFIX+".storagetypes";

	public static final String INFO_PROVIDER_SUFFIX=".infoprovider";

	public StorageDescriptionRP(Resource parent, String xnjsReference){
		super(parent);
		xml=new StorageDescriptionDocument[0];
		this.xnjsReference=xnjsReference;
	}

	@Override
	public StorageDescriptionDocument[] getXml() {
		return xml;
	}

	@Override
	public ResourceProperty<Object> update(){
		if(mustUpdate()){
			String typesP=parentWSResource.getKernel().getProperty(DEFINED_TYPES, "DEFAULT");
			if(typesP==null)typesP="DEFAULT";

			String[] types=typesP.trim().split(" +");
			xml=new StorageDescriptionDocument[types.length];
			int i=0;
			for(String type: types){
				xml[i]=getStorageDescription(type);
				i++;
			}
		}
		return this;
	}

	protected StorageDescriptionDocument getStorageDescription(String type){
		StorageDescriptionDocument sdd=StorageDescriptionDocument.Factory.newInstance();
		StorageDescriptionType sType=sdd.addNewStorageDescription();
		sType.setStorageBackendType(type);
		FileSystemDocument fs=getFileSystemDescription(type);
		if(fs!=null){
			sType.setFileSystem(fs.getFileSystem());
		}
		return sdd;
	}

	protected FileSystemDocument getFileSystemDescription(String type){
		Kernel kernel=parentWSResource.getKernel();
		String infoP=kernel.getProperty(StorageFactoryImpl.PREFIX+"."+type+INFO_PROVIDER_SUFFIX);
		if(infoP== null && "DEFAULT".equals(type)){
			infoP=DefaultStorageInfoProvider.class.getName();
		}
		if(infoP!=null){
			try{
				StorageInfoProvider infoProvider=(StorageInfoProvider)kernel.load(Class.forName(infoP));
				//check if we have an info provider
				Client client=AuthZAttributeStore.getClient();
				FileSystemDocument fs=infoProvider.getInformation(type,client,xnjsReference);
				return fs;
			}
			catch(Exception ex){
				LogUtil.logException("Can't get storage information for <"+type+">", ex, logger);
			}
		}
		return null;
	}

	protected boolean mustUpdate(){
		return lastPropertyUpdate<=parentWSResource.getKernel().getLastConfigFileChangeTime();
	}

}
