package de.fzj.unicore.uas.metadata;

import org.apache.log4j.Logger;

import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.xnjs.io.IStorageAdapter;
import eu.unicore.security.util.Log;

/**
 * factory and configuration class for the metadata support
 * 
 * @author schuller
 */
public class MetadataSupport {

	private static final Logger logger=Log.getLogger(Log.SERVICES, MetadataSupport.class);

	private static Class<? extends MetadataManager>managerClass;

	private MetadataSupport(){}

	/**
	 * the property name for configuring the "default" MetadataManager implementation
	 */
	public static final String METADATA_MANAGER_CLASSNAME="uas.metadata.manager.class";

	public static synchronized void setManagerClass(Class<? extends MetadataManager>managerClass){
		MetadataSupport.managerClass=managerClass;
		logger.info("Using MetadataManager class: "+managerClass.getName());
	}

	/**
	 * get a new instance of the {@link MetadataManager}<br/>
	 * 
	 * The class to be used can be set using either a call to setManagerClass(), or by setting the  
	 * property {@link #METADATA_MANAGER_CLASSNAME}
	 * 
	 * @return
	 * @throws Exception
	 */
	@SuppressWarnings("unchecked")
	public static synchronized MetadataManager getManager(Kernel kernel){
		if(managerClass==null){
			try{
				String mmP=kernel.getProperty(METADATA_MANAGER_CLASSNAME);
				if(mmP!=null){
					managerClass=(Class<? extends MetadataManager>) Class.forName(mmP);
					logger.info("Using MetadataManager class: "+mmP);
				}
			}
			catch(Exception ex){
				Log.logException("Cannot setup metadata manager.", ex, logger);
			}
		}
		
		if(managerClass!=null){
			try{
				MetadataManager mm=kernel.load(managerClass);
				return mm;
			}catch(Exception ex){
				Log.logException("Cannot instatiate metadata manager from class "+managerClass.getName(), ex, logger);
			}
		}
		return null;
	}
	
	public static synchronized MetadataManager getManager(Kernel kernel, IStorageAdapter storage, String uniqueID){
		MetadataManager mm=MetadataSupport.getManager(kernel);
		if(mm instanceof StorageMetadataManager){
			StorageMetadataManager smm=(StorageMetadataManager)mm;
			smm.setStorageAdapter(storage, uniqueID);
		}
		return mm;
	}
}
