package de.fzj.unicore.uas.security;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.security.KeyStore;
import java.util.Properties;

import javax.security.auth.x500.X500Principal;

import junit.framework.TestCase;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.security.UASSecurityProperties;

public class TestProxyGenerator extends TestCase {


	private static String secPropsOut=
		ISecurityProperties.WSRF_SSL + "=true"+"\n"+
		ISecurityProperties.WSRF_SSL_CLIENTAUTH + "=true" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYPASS + "=the!user"+"\n"+

		ISecurityProperties.WSRF_SSL_TRUSTSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTPASS + "=the!user"+"\n"+

		""
		;

	public void testGenerateJKS()throws Exception{
		ISecurityProperties sec=new UASSecurityProperties(new Properties(),
				new ByteArrayInputStream(secPropsOut.getBytes()));
		ProxyCertificate proxy=ProxyCertificate.getInstance(sec, false, 48*3600*1000, 1024);
		assertNotNull(proxy);
		File file=File.createTempFile("proxy", ".jks");
		//file.deleteOnExit();
		proxy.writeJKS(file.getAbsolutePath());
		assertTrue(file.exists());
		KeyStore ks=KeyStore.getInstance("jks");
		InputStream is=new FileInputStream(file);
		ks.load(is, ProxyCertificate.PROXY_JKS_PASSWORD.toCharArray());
		assertTrue(ks.isKeyEntry(ProxyCertificate.PROXY_JKS_ALIAS));
	}

	public void testProxyGenerator()throws Exception{
		ISecurityProperties sec=new UASSecurityProperties(new Properties(),
				new ByteArrayInputStream(secPropsOut.getBytes()));
		ProxyCertificate proxy=ProxyCertificate.getInstance(sec, false, 10*3600*1000, 1024);
		assertNotNull(proxy);
		assertEquals(1024,proxy.getKeyLength());
		X500Principal proxyDN=proxy.getProxyCertificate().getSubjectX500Principal();
		X500Principal issuerDN=proxy.getProxyCertificate().getIssuerX500Principal();
		System.out.println("proxy=    "+proxyDN);
		System.out.println("issuer=   "+issuerDN);
		System.out.println("original= "+proxy.getUserCertificate().getSubjectX500Principal().toString());
		System.out.println(proxy.getPEMEncoded());
	}
	
	private static String secPropsOut2=
		ISecurityProperties.WSRF_SSL + "=true"+"\n"+
		ISecurityProperties.WSRF_SSL_CLIENTAUTH + "=true" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYSTORE + "=src/test/resources/dsa-key.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_KEYPASS + "=123"+"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTSTORE + "=src/test/resources/dsa-key.jks" +"\n"+
		ISecurityProperties.WSRF_SSL_TRUSTPASS + "=123"+"\n";
	
	public void testProxyGeneratorWithDSAKey() throws Exception{
		ISecurityProperties sec=new UASSecurityProperties(new Properties(),
				new ByteArrayInputStream(secPropsOut2.getBytes()));
		try{
			ProxyCertificate proxy=ProxyCertificate.getInstance(sec, false, 3600*1000, 1024);
			assertNotNull(proxy);
			assertEquals(1024,proxy.getKeyLength());
		}catch(Exception ex){
			ex.printStackTrace();
			fail(ex.getMessage());
		}
	}

}
	
