package de.fzj.unicore.uas.testsuite;

import static org.junit.Assert.assertTrue;

import java.util.Calendar;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.ResourcesDocument;
import org.unigrids.x2006.x04.services.reservation.ReservationPropertiesDocument;
import org.unigrids.x2006.x04.services.tss.SubmitDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.ReservationClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.wsrflite.Kernel;

public class TestReservation extends RunDate {
	String url;
	EndpointReferenceType epr;

	@Override
	public void testRunJob()throws Exception{

		url=kernel.getProperty(Kernel.WSRF_BASEURL);
		tssepr=EndpointReferenceType.Factory.newInstance();
		tssepr.addNewAddress().setStringValue(url+"/"+UAS.TSS+"?res=default_targetsystem");
		tss=new TSSClient(tssepr,kernel.getSecurityProperties());
		//as our config uses a dummy reservation module, we support reservations
		assertTrue(tss.supportsReservation());
		ResourcesDocument resources=ResourcesDocument.Factory.newInstance();
		resources.addNewResources().addNewIndividualCPUTime().addNewExact().setDoubleValue(7200);
		Calendar startTime=Calendar.getInstance();
		startTime.set(2010, 11, 24);
		ReservationClient reservation=tss.createReservationClient(resources, startTime);
		ReservationPropertiesDocument props=reservation.getResourcePropertiesDocument();
		String resID=props.getReservationProperties().getReservationReference();
		System.out.println("+++ reservation ID="+resID);

		//check we have a reference in the TSS properties
		assertTrue(1==tss.getReservations().size());

		//now submit a job using the reservation
		JobDefinitionDocument jdd=getJob();
		SubmitDocument req=SubmitDocument.Factory.newInstance();
		req.addNewSubmit().setJobDefinition(jdd.getJobDefinition());
		JobClient job=reservation.submit(req);
		job.waitUntilReady(180*1000);
		job.start();
		job.waitUntilDone(180*1000);

		//now delete the reservation
		reservation.destroy();

		//and check the reservation list on the TSS is updated
		tss.setUpdateInterval(0);
		assertTrue(0==tss.getReservations().size());
	}

}
