package de.fzj.unicore.uas.functional;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Random;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.unigrids.services.atomic.types.ProtocolType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.UDTClient;
import de.fzj.unicore.uas.fts.udt.UDTSession;
import de.fzj.unicore.xnjs.XNJS;

public class RunClientServerTransfer extends AbstractJobRun {

	String url;
	EndpointReferenceType tssepr;

	public RunClientServerTransfer(){
	}

	@Override
	public String getDescription() {
		return "Tests the client-server transfer.";
	}

	@Override
	protected void beforeStart(JobClient jms) throws Exception {
		
		String protocol="UDT";
		System.out.println("\n\nTesting client-server transfer using <"+protocol+">\n\n");
		System.out.println();
		
		runUpload(protocol, jms);
		//runBigUpload(protocol, jms);
		//runDownload(protocol, jms);
	}
	
	private static XNJS clientXNJS=null;

	protected void runUpload(String protocol, JobClient jms)throws Exception{
		//start "client-side" xnjs instance
		if(clientXNJS==null)clientXNJS=new XNJS("src/test/resources/xnjs.xml");
		UDTClient ftc=(UDTClient)jms.getUspaceClient().getImport("/testfile",ProtocolType.UDT);
		ftc.setExportMode(true);
		ftc.setMyHost(UDTSession.getLocalhostAddress());
		ftc.setConfiguration(clientXNJS.getConfig());
		
		//write test data
		File tmp=File.createTempFile("udt", "test");
		FileOutputStream fos=new FileOutputStream(tmp);
		String data="this is some test data";
		fos.write(data.getBytes());
		fos.close();
		ftc.setLocalFile(tmp.getAbsolutePath());
		System.out.println("remote host: '"+ftc.getRemoteHost()+"'");
		System.out.println("is sender  : '"+ftc.getRemoteHostIsSender()+"'");
		ftc.run();
		
		//send data
		FileInputStream fis=new FileInputStream(tmp);
		ftc.writeAllData(fis);
		fis.close();
		
		while(true){
			Thread.sleep(500);
			ftc.checkError();
			long size=ftc.getSourceFileSize();
			long transferred=ftc.getTransferredBytes();
			System.out.println("Size of file to transfer: "+size);
			System.out.println("Bytes transferred:        "+transferred);
			if(transferred>0 && transferred>=size)break;
		}
		ftc.destroy();
		//check data
		ByteArrayOutputStream bos=new ByteArrayOutputStream();
		jms.getUspaceClient().getExport("/testfile").readAllData(bos);
		String x = bos.toString();
		assertTrue(data.equals(x));
	}

	protected void runBigUpload(String protocol, JobClient jms)throws Exception{
		//start "client-side" xnjs instance
		if(clientXNJS==null)clientXNJS=new XNJS("src/test/resources/xnjs.xml");
		UDTClient ftc=(UDTClient)jms.getUspaceClient().getImport("/testfile2",ProtocolType.UDT);
		ftc.setExportMode(true);
		ftc.setMyHost(UDTSession.getLocalhostAddress());
		ftc.setConfiguration(clientXNJS.getConfig());
		File tmp=File.createTempFile("udt", "test");
		FileOutputStream fos=new FileOutputStream(tmp);
		Random r=new Random();
		int NUM=10;
		int CHUNKSIZE=128000;
		byte[] data=new byte[CHUNKSIZE];
		for(int i=0;i<NUM;i++){
			r.nextBytes(data);
			fos.write(data);
		}
		fos.close();
		ftc.setLocalFile(tmp.getAbsolutePath());
		System.out.println("remote host: '"+ftc.getRemoteHost()+"'");
		System.out.println("is sender  : '"+ftc.getRemoteHostIsSender()+"'");
		long start=System.currentTimeMillis();
		ftc.run();
		while(true){
			Thread.sleep(3000);
			ftc.checkError();
			long size=ftc.getSourceFileSize();
			long transferred=ftc.getTransferredBytes();
			System.out.println("Size of file to transfer: "+size);
			System.out.println("Bytes transferred:        "+transferred);
			if(transferred>0 && transferred>=size)break;
		}
		ftc.destroy();
		long end=System.currentTimeMillis();
		long size=NUM*CHUNKSIZE;
		double rate=((double)size/(double)(end-start))/1e3;
		System.out.println("Size: "+size+" Time: "+(end-start)/1000+" Rate: "+rate+" MB/sec.");
		tmp.delete();
	}

	protected void runDownload(String protocol, JobClient jms)throws Exception{
		//start "client-side" xnjs instance
		if(clientXNJS==null)clientXNJS=new XNJS("src/test/resources/xnjs.xml");
		//upload data
		String data="this is some test data";
		ByteArrayInputStream bis=new ByteArrayInputStream(data.getBytes());
		jms.getUspaceClient().getImport("/testfile3",ProtocolType.BFT).writeAllData(bis);
		
		UDTClient ftc=(UDTClient)jms.getUspaceClient().getExport("/testfile3",ProtocolType.UDT);
		ftc.setExportMode(false);
		ftc.setMyHost(UDTSession.getLocalhostAddress());
		ftc.setConfiguration(clientXNJS.getConfig());
		File tmp=File.createTempFile("udt", "test");
		ftc.setLocalFile(tmp.getAbsolutePath());
		System.out.println("remote host: '"+ftc.getRemoteHost()+"'");
		System.out.println("is sender  : '"+ftc.getRemoteHostIsSender()+"'");
		ftc.run();
		while(true){
			Thread.sleep(500);
			ftc.checkError();
			long size=ftc.getSourceFileSize();
			long transferred=ftc.getTransferredBytes();
			System.out.println("Size of file to transfer: "+size);
			System.out.println("Bytes transferred:        "+transferred);
			if(transferred>=size)break;
		}
		ftc.destroy();
		
		//check data
		FileInputStream fis=new FileInputStream(tmp);
		byte[]test=new byte[data.length()];
		fis.read(test);
		assertTrue(new String(test).equals(data));
		fis.close();
	}
	
	
	@Override
	protected void onError(){

	}


	@Override
	protected void onFinish(JobClient jms) throws Exception {

	}

	@Override
	protected JobDefinitionDocument getJob() {
		JobDefinitionDocument jdd=JobDefinitionDocument.Factory.newInstance();
		ApplicationType app=jdd.addNewJobDefinition().addNewJobDescription().addNewApplication();
		app.setApplicationName("Date");
		app.setApplicationVersion("1.0");
		return jdd;
	}
}
